s3

package
v0.3.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 24, 2026 License: MPL-2.0 Imports: 44 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

Types

type Backend

type Backend struct {
	// contains filtered or unexported fields
}

func (*Backend) ConfigSchema

func (b *Backend) ConfigSchema() *configschema.Block

ConfigSchema returns a description of the expected configuration structure for the receiving backend. This structure is mirrored by the encryption aws_kms key provider and should be kept in sync.

func (*Backend) Configure

func (b *Backend) Configure(ctx context.Context, obj cty.Value) tfdiags.Diagnostics

Configure uses the provided configuration to set configuration fields within the backend.

The given configuration is assumed to have already been validated against the schema returned by ConfigSchema and passed validation via PrepareConfig.

func (*Backend) DeleteWorkspace

func (b *Backend) DeleteWorkspace(ctx context.Context, name string, _ bool) error

func (*Backend) PrepareConfig

func (b *Backend) PrepareConfig(obj cty.Value) (cty.Value, tfdiags.Diagnostics)

PrepareConfig checks the validity of the values in the given configuration, and inserts any missing defaults, assuming that its structure has already been validated per the schema returned by ConfigSchema.

func (*Backend) StateMgr

func (b *Backend) StateMgr(ctx context.Context, name string) (statemgr.Full, error)

func (*Backend) Workspaces

func (b *Backend) Workspaces(ctx context.Context) ([]string, error)

type RemoteClient

type RemoteClient struct {
	// contains filtered or unexported fields
}

func (*RemoteClient) Delete

func (c *RemoteClient) Delete(ctx context.Context) error

func (*RemoteClient) Get

func (c *RemoteClient) Get(ctx context.Context) (payload *remote.Payload, err error)

func (*RemoteClient) IsLockingEnabled

func (c *RemoteClient) IsLockingEnabled() bool

func (*RemoteClient) Lock

func (c *RemoteClient) Lock(ctx context.Context, info *statemgr.LockInfo) (string, error)

func (*RemoteClient) Put

func (c *RemoteClient) Put(ctx context.Context, data []byte) error

func (*RemoteClient) Unlock

func (c *RemoteClient) Unlock(ctx context.Context, id string) error

Jump to