Documentation
¶
Index ¶
- func New(enc encryption.StateEncryption) backend.Backend
- type Backend
- func (b *Backend) ConfigSchema() *configschema.Block
- func (b *Backend) Configure(ctx context.Context, obj cty.Value) tfdiags.Diagnostics
- func (b *Backend) DeleteWorkspace(ctx context.Context, name string, _ bool) error
- func (b *Backend) PrepareConfig(obj cty.Value) (cty.Value, tfdiags.Diagnostics)
- func (b *Backend) StateMgr(ctx context.Context, name string) (statemgr.Full, error)
- func (b *Backend) Workspaces(ctx context.Context) ([]string, error)
- type RemoteClient
- func (c *RemoteClient) Delete(ctx context.Context) error
- func (c *RemoteClient) Get(ctx context.Context) (payload *remote.Payload, err error)
- func (c *RemoteClient) IsLockingEnabled() bool
- func (c *RemoteClient) Lock(ctx context.Context, info *statemgr.LockInfo) (string, error)
- func (c *RemoteClient) Put(ctx context.Context, data []byte) error
- func (c *RemoteClient) Unlock(ctx context.Context, id string) error
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func New ¶
func New(enc encryption.StateEncryption) backend.Backend
Types ¶
type Backend ¶
type Backend struct {
// contains filtered or unexported fields
}
func (*Backend) ConfigSchema ¶
func (b *Backend) ConfigSchema() *configschema.Block
ConfigSchema returns a description of the expected configuration structure for the receiving backend. This structure is mirrored by the encryption aws_kms key provider and should be kept in sync.
func (*Backend) Configure ¶
Configure uses the provided configuration to set configuration fields within the backend.
The given configuration is assumed to have already been validated against the schema returned by ConfigSchema and passed validation via PrepareConfig.
func (*Backend) DeleteWorkspace ¶
func (*Backend) PrepareConfig ¶
PrepareConfig checks the validity of the values in the given configuration, and inserts any missing defaults, assuming that its structure has already been validated per the schema returned by ConfigSchema.
type RemoteClient ¶
type RemoteClient struct {
// contains filtered or unexported fields
}
func (*RemoteClient) IsLockingEnabled ¶
func (c *RemoteClient) IsLockingEnabled() bool
Click to show internal directories.
Click to hide internal directories.